@@ -58,6 +58,7 @@ public class Form : ContainerControl
 	internal static Form activeForm;
 	private bool showInTaskbar;
 	private bool controlBox;
+	private bool loaded; /* whether we have sent OnLoad or not */
 	private MdiClient mdiClient;
 	
 	// Constructor.
@@ -66,6 +67,7 @@ public class Form : ContainerControl
 				visible = false;
 				autoScale = true;
 				topLevel = true;
+				loaded=false;
 				borderStyle = FormBorderStyle.Sizable;
 				mdiChildren = new Form [0];
 				ownedForms = new Form [0];
@@ -120,12 +122,12 @@ public class Form : ContainerControl
 			{
 				get
 				{
-					// TODO
+					// Implement
 					return false;
 				}
 				set
 				{
-					// TODO
+					// Implement
 				}
 			}
 	public bool AutoScale
@@ -251,8 +253,10 @@ public class Form : ContainerControl
 				{
 					if(borderStyle != value)
 					{
+						Size oldSize = ClientSize;
 						borderStyle = value;
 						SetWindowFlag(0, true);
+						ClientSize = oldSize;
 					}
 				}
 			}
@@ -335,12 +339,12 @@ public class Form : ContainerControl
 			{
 				get
 				{
-					// TODO
+					// Implement
 					return Rectangle.Empty;
 				}
 				set
 				{
-					// TODO
+					// Implement
 				}
 			}
 	public Size MaximumSize
@@ -530,7 +534,7 @@ public class Form : ContainerControl
 				}
 				set
 				{
-					// TODO: update the owned child list
+					// Fix: update the owned child list
 					owner = value;
 				}
 			}
@@ -656,7 +660,7 @@ public class Form : ContainerControl
 				{
 					return base.CreateToolkitWindow(parent);
 				}
-				
+
 				CreateParams cp = CreateParams;
 
 				// Create the window and set its initial caption.
@@ -747,7 +751,10 @@ public class Form : ContainerControl
 				{
 					case FormBorderStyle.None:
 					{
-						flags = (ToolkitWindowFlags.ShowInTaskbar);
+						bool topMost = ((flags & ToolkitWindowFlags.TopMost) != 0);
+						flags = ToolkitWindowFlags.ShowInTaskbar;
+						if (topMost)
+							flags |= ToolkitWindowFlags.TopMost;
 					}
 					break;
 
@@ -846,7 +853,6 @@ public class Form : ContainerControl
 	[TODO]
 	public static SizeF GetAutoScaleSize(Font font)
 			{
-				// TODO
 				return SizeF.Empty;
 			}
 
@@ -1189,7 +1195,11 @@ public class Form : ContainerControl
 	protected override void OnCreateControl()
 			{
 				base.OnCreateControl();
-				OnLoad(EventArgs.Empty);
+				if(loaded == false && toolkitWindow.IsMapped)
+				{
+					loaded = true;
+					OnLoad(EventArgs.Empty);
+				}
 			}
 
 	// Emit the "Deactivate" event.
@@ -1377,6 +1387,12 @@ public class Form : ContainerControl
 	protected override void OnVisibleChanged(EventArgs e)
 			{
 				base.OnVisibleChanged(e);
+				if(loaded == false && 
+						toolkitWindow != null && toolkitWindow.IsMapped)
+				{
+					loaded = true;
+					OnLoad(EventArgs.Empty);
+				}
 			}
 
 	// Process a command key.
@@ -1446,7 +1462,6 @@ public class Form : ContainerControl
 	[TODO]
 	protected override void ScaleCore(float dx, float dy)
 			{
-				// TODO
 				base.ScaleCore(dx, dy);
 			}
 
@@ -1541,21 +1556,33 @@ public class Form : ContainerControl
 			{
 				get
 				{
-					int leftAdjust, topAdjust, rightAdjust, bottomAdjust;
-					ToolkitManager.Toolkit.GetWindowAdjust
-						(out leftAdjust, out topAdjust,
-						out rightAdjust, out bottomAdjust, GetFullFlags());
-					return new Size(leftAdjust + rightAdjust, topAdjust + bottomAdjust);
+					if(FormBorderStyle != FormBorderStyle.None)
+					{
+						int leftAdjust, topAdjust, rightAdjust, bottomAdjust;
+						ToolkitManager.Toolkit.GetWindowAdjust
+							(out leftAdjust, out topAdjust,
+							out rightAdjust, out bottomAdjust, GetFullFlags());
+						return new Size(leftAdjust + rightAdjust, topAdjust + bottomAdjust);
+					}
+					else
+					{
+						return Size.Empty;
+					}
 				}
 			}
 	
 	// Convert a client size into a window bounds size.
 	internal override Size ClientToBounds(Size size)
 			{
-				int leftAdjust, topAdjust, rightAdjust, bottomAdjust;
-				ToolkitManager.Toolkit.GetWindowAdjust
-					(out leftAdjust, out topAdjust,
-					out rightAdjust, out bottomAdjust, GetFullFlags());
+				int leftAdjust = 0, topAdjust = 0, rightAdjust = 0, bottomAdjust = 0;
+
+				if (FormBorderStyle != FormBorderStyle.None)
+				{
+					ToolkitManager.Toolkit.GetWindowAdjust
+						(out leftAdjust, out topAdjust,
+						out rightAdjust, out bottomAdjust, GetFullFlags());
+				}
+				
 				if (Menu != null)
 					topAdjust += SystemInformation.MenuHeight;
 				return new Size(size.Width + leftAdjust + rightAdjust,
@@ -1576,7 +1603,7 @@ public class Form : ContainerControl
 				if(!(args.Cancel))
 				{
 					OnClosed(EventArgs.Empty);
-					DestroyHandle();
+					Dispose();
 				}
 			}
 
@@ -1628,13 +1655,11 @@ public class Form : ContainerControl
 		[TODO]
 		public override void Add(Control control)
 				{
-					// TODO
 					base.Add(control);
 				}
 		[TODO]
 		public override void Remove(Control control)
 				{
-					// TODO
 					base.Remove(control);
 				}
 
@@ -1669,6 +1694,7 @@ public class Form : ContainerControl
 				base.OnMouseMove (e);
 			}
 
+	[TODO]
 	public bool ControlBox
 			{
 				get
@@ -1680,7 +1706,7 @@ public class Form : ContainerControl
 					if (value != controlBox)
 					{
 						controlBox = value;
-						//TODO
+						// Implement
 					}
 				}
 			}
