@@ -77,6 +77,8 @@ namespace System.Windows.Forms
 		private const int minimumTabSize = 45;
 		// Indent of the actual tabs from the side of the control
 		private const int indent = 2;
+		// Used to track first paint call, hack to mimmick MS implementation
+		// of setting first added tab as active
 
 		// Tab events
 		public event DrawItemEventHandler DrawItem;
@@ -120,10 +122,11 @@ namespace System.Windows.Forms
 
 		protected virtual void OnSelectedIndexChanged(EventArgs e)
 		{
-			if (selectedIndex == -1 || selectedIndex >= TabCount)
-			{
-				return;
-			}
+			//redundant check, is done is SelectedIndex already
+			//if (selectedIndex == -1 || selectedIndex >= TabCount)
+			//{
+			//	return;
+			//}
 			SuspendLayout();
 			if (prevSelectedIndex > -1)
 			{
@@ -273,12 +276,12 @@ namespace System.Windows.Forms
 				}
 				else
 				{
-					return tabPageCollection[selectedIndex] as TabPage;
+					return (TabPage)tabPageCollection[selectedIndex];
 				}
 			}
 			set
 			{
-				SelectedIndex = tabPageCollection.IndexOf(value);
+				selectedIndex = tabPageCollection.IndexOf(value);
 			}
 		}
 
@@ -327,7 +330,20 @@ namespace System.Windows.Forms
 			return new TabChildControls( this );
 		}
 
-		
+		public override String ToString()
+		{
+			string ret =  base.ToString() + ", TabPages.Count: " + TabCount.ToString();
+			if(TabPages.Count > 0)
+			{
+				for(int cnt = 0; cnt < TabPages.Count; cnt++)
+				{
+					ret += ", TabPages[" + cnt + "]: TabPage: " + TabPages[cnt].Text;
+				}
+			}
+
+			return ret;
+		}
+	
 		// Collection of child control TabPages.
 		public class TabChildControls : Control.ControlCollection
 		{
@@ -341,20 +357,14 @@ namespace System.Windows.Forms
 
 			public override void Add(Control control)
 			{
-				control.Visible = false;
 				base.Add(control);
 				tabOwner.InvalidateTabs	();
 				tabOwner.SetTabPageBounds();
-				if (tabOwner.SelectedTab == control)
-				{
-					control.Visible = true;
-				}
 			}
 
 			[TODO]
 			public override void Remove(Control control)
 			{
-				// TODO
 				base.Remove(control);
 			}
 
@@ -380,7 +390,7 @@ namespace System.Windows.Forms
 				}
 				set
 				{
-					// TODO
+					// Handle
 				}
 			}
 
@@ -530,6 +540,7 @@ namespace System.Windows.Forms
 
 		protected override void OnPaint(PaintEventArgs e)
 		{
+
 			Draw(e.Graphics);
 			// Draw the visible TabPage (child controls)
 			base.OnPaint (e);
@@ -590,7 +601,7 @@ namespace System.Windows.Forms
 			{
 				for( int i = 0; i < tabPageCollection.Count; i++ )
 				{
-					if (row == PositionInfo.positions[i].row && i != selectedIndex)
+					if (row == PositionInfo.positions[i].row && i != SelectedIndex)
 					{
 						Rectangle bounds = GetTabRect(i);
 						// Remove bottom line off bounds if not selected so border isnt covered
@@ -600,17 +611,17 @@ namespace System.Windows.Forms
 					}
 				}
 			}
-			if (selectedIndex > -1 && selectedIndex < TabPages.Count)
+			if (SelectedIndex < TabPages.Count)
 			{
-				Rectangle bounds = GetTabRect(selectedIndex);
+				Rectangle bounds = GetTabRect(SelectedIndex);
 				g.SetClip(new Rectangle(bounds.Left, bounds.Top, bounds.Width, bounds.Height), Drawing.Drawing2D.CombineMode.Intersect);
 				
 				DrawItemState state;
 				if (Focused)
-					state = DrawItemState.Focus;
+					state = DrawItemState.Focus;// | DrawItemState.Selected;
 				else
-					state = DrawItemState.None;
-				OnDrawItem( new DrawItemEventArgs(g, Font, bounds, selectedIndex, state, ForeColor, BackColor));
+					state = DrawItemState.Default; //DrawItemState.Selected;
+				OnDrawItem( new DrawItemEventArgs(g, Font, bounds, SelectedIndex, state, ForeColor, BackColor));
 			}
 
 		}
@@ -660,12 +671,13 @@ namespace System.Windows.Forms
 								SizeTabsFillRight ( ref tabs, graphics, rowWidth, out maxRow );
 						}
 						// Do we need to move the row that was selected to the bottom of the tabs?
-						if (selectedIndex > -1 && selectedIndex < TabCount)
-						{
+						// why is bounds checked here again? its redundant.
+						//if (SelectedIndex < TabCount)
+						//{
 							// Check to see if we have selected a tab that isnt on the last row and move the tab row down
-							if (tabs[selectedIndex].row != maxRow)
-								RowToBottom(ref tabs, tabs[selectedIndex].row, maxRow);
-						}
+							if (tabs[SelectedIndex].row != maxRow)
+								RowToBottom(ref tabs, tabs[SelectedIndex].row, maxRow);
+						//}
 
 						// Find the actual bounds
 						LayoutTabBounds( ref tabs, rowWidth );
@@ -907,13 +919,13 @@ namespace System.Windows.Forms
 				{
 					// Move the tabs thats selected by 1
 					int selDelta = 0;
-					if (i != selectedIndex)
+					if (i != SelectedIndex)
 						selDelta = 1;
 
 					TabPosition tab = tabs[i];
 					int width = tab.bounds.Width + (1-selDelta)*2*indent;
 					// Draw the left edge of tabs that begin a row or are selected
-					if (tab.bounds.Left - leftOffset == 0 || selectedIndex == i)
+					if (tab.bounds.Left - leftOffset == 0 || SelectedIndex == i)
 						tab.leftExposed = true;
 					tab.bounds = new Rectangle(tab.bounds.Left + indent - (1-selDelta)*2 - leftOffset, yDirection *((tab.row + extraRow ) * down + selDelta) + top, width, down + 1 - selDelta );
 					tabs[i] = tab;
@@ -991,6 +1003,10 @@ namespace System.Windows.Forms
 			}
 			set
 			{
+				if (value < 0 || value >= tabPageCollection.Count)
+				{
+					throw new IndexOutOfRangeException(value.ToString());
+				}
 				if (value != selectedIndex)
 				{
 					prevSelectedIndex = selectedIndex;
@@ -1000,13 +1016,14 @@ namespace System.Windows.Forms
 			}
 		}
 
+		[TODO]
 		// This occurs for each tab needing to be drawn
 		protected virtual void OnDrawItem( DrawItemEventArgs e )
 		{
 			e.DrawBackground();
 			Rectangle borderBounds = new Rectangle(e.Bounds.Left, e.Bounds.Top, e.Bounds.Width, e.Bounds.Height+2);
 			
-			//TODO: If Appearance is tab then this ok, need to do for button & flat button
+			// Fix: If Appearance is tab then this ok, need to do for button & flat button
 			// Draw tab edging & clip a border edge above the bottom
 			
 			DrawTab( e.Graphics, borderBounds, alignment, PositionInfo.positions[e.Index].leftExposed );
@@ -1014,7 +1031,7 @@ namespace System.Windows.Forms
 			// Owner Draw does their own drawing
 			if (drawMode == TabDrawMode.Normal)
 			{
-				//TODO: If imageindex & imagelist then draw
+				// Fix: If imageindex & imagelist then draw
 				
 				Rectangle textBounds = new Rectangle(e.Bounds.Left + tabTextWidthMargin, e.Bounds.Top + tabTextHeightMargin, e.Bounds.Width - tabTextWidthMargin*2, e.Bounds.Height - tabTextHeightMargin * 2);
 				Rectangle focusBounds = textBounds;
@@ -1022,7 +1039,7 @@ namespace System.Windows.Forms
 
 				if ((e.State & DrawItemState.Focus)>0)
 					ControlPaint.DrawFocusRectangle(e.Graphics, focusBounds, ForeColor, BackColor);
-				//TODO: Draw disabled
+				// Fix: Draw disabled
 				DrawText( e.Graphics, tabPageCollection[e.Index].Text, textBounds, e.ForeColor, e.BackColor, hotTrack && e.Index == hotTrackIndex );
 				
 			}
