@@ -1044,8 +1044,7 @@ public class DefaultThemePainter : IThemePainter
 										int steps, int step,
 										int value, bool enabled)
 			{
-				// TODO : draw disabled mode progressbar
-				// TODO : handle large no of blocks ie merge cases
+				// TODO : handle large number of blocks ie merge cases
 				int blockWidth, blockHeight, xSpacing, ySpacing;
 				DrawBorder3D(graphics,x,y,width,height, 
 							 SystemColors.InactiveBorder,
@@ -1538,13 +1537,28 @@ public class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw an image in its disabled state.
-	[TODO]
 	public virtual void DrawImageDisabled
 				(Graphics graphics, Image image,
 				 int x, int y, Color background)
 			{
-				// TODO: create the greyed version of the image.
-				graphics.DrawImage(image, x, y);
+				Imaging.ColorMatrix colorMatrix = new Imaging.ColorMatrix(new float[][] { new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0.5f,0.5f,0.5f,0,0 },
+											new float[]{ 0,0,0,1,0,0 },
+											new float[]{ 0,0,0,0,1,0 },
+											new float[]{ 0,0,0,0,0,1 }
+											}
+									);
+
+				Imaging.ImageAttributes imageAttributes = new Imaging.ImageAttributes();
+				imageAttributes.SetColorMatrix(colorMatrix);
+				graphics.DrawImage(image,
+						new Rectangle(x, y, image.Width, image.Height),
+						0, 0,
+						image.Width,
+						image.Height,
+						GraphicsUnit.Pixel,
+						imageAttributes);
 			}
 
 	// Draw a locked selection frame.
@@ -1769,6 +1783,9 @@ public class DefaultThemePainter : IThemePainter
 				 Rectangle decrement, bool decDown,
 				 Rectangle increment, bool incDown)
 			{
+				
+				               
+	
 				// fill in the background
 				//graphics.FillRectangle(backgroundBrush, bounds);
 				if (hatchBrush != null && (foreColor != hatchForeColor ||
@@ -1854,7 +1871,7 @@ public class DefaultThemePainter : IThemePainter
 				ButtonState barState = enabled ?
 				                       ButtonState.Normal :
 				                       ButtonState.Inactive;
-
+				
 				// draw the scroll bar
 				if (track.Height > 0 && bar.IntersectsWith(drawBounds))
 					DrawButton(graphics,
@@ -1862,7 +1879,7 @@ public class DefaultThemePainter : IThemePainter
 				           bar.Width, bar.Height,
 				           barState,
 				           foreColor, backColor,
-				           false);
+				           false);   
 			}
 
 	// Draw a scroll button control.
@@ -1871,7 +1888,9 @@ public class DefaultThemePainter : IThemePainter
 				 ScrollButton button, ButtonState state,
 				 Color foreColor, Color backColor)
 			{
-				// draw the button border
+				
+			        // draw the button border
+				
 				DrawButton(graphics,
 				           x, y, width, height,
 				           state,
@@ -1897,28 +1916,59 @@ public class DefaultThemePainter : IThemePainter
 				}
 
 				// Draw the inactive drop-down glyph for the button.
+				
+				if(width==0)width=-1;
+				
 				if((state & ButtonState.Inactive) != 0)
 				{
+				
 					switch (button)
 					{
 						case ScrollButton.Up:
 						{
-							DrawGlyph(graphics, x + 1, y + 1, width, height,
+							if(width<=20) // if there is not enough space we only draw a point
+							{
+								DrawGlyph(graphics, x + 1, y + 1, width - 3, height,
+								 	Glyphs.up_arrow_bits,
+								  	Glyphs.up_arrow_width / 2,
+								  	Glyphs.up_arrow_height / 2, inactiveColor);
+						
+							}
+
+							else 
+							{ 
+						
+								DrawGlyph(graphics, x + 1, y + 1, width, height,
 									  Glyphs.up_arrow_bits,
 									  Glyphs.up_arrow_width,
 									  Glyphs.up_arrow_height, inactiveColor);
+							}
 						}
 						break;
 	
 						case ScrollButton.Down:
 						{
-							DrawGlyph(graphics, x + 1, y + 1, width, height,
+							if(width<=20)  // if there is not enough space we only draw a point 
+							{
+								DrawGlyph(graphics, x + 1, y + 1, width - 3, height,
+									  Glyphs.up_arrow_bits,
+								  	  Glyphs.up_arrow_width / 2,
+								 	  Glyphs.up_arrow_height / 2, inactiveColor);
+						
+							}
+
+							else 
+							{ 
+						
+								DrawGlyph(graphics, x + 1, y + 1, width, height,
 									  Glyphs.down_arrow_bits,
 									  Glyphs.down_arrow_width,
 									  Glyphs.down_arrow_height, inactiveColor);
+						
+							}
 						}
 						break;
-	
+	                                        
 						case ScrollButton.Left:
 						{
 							DrawGlyph(graphics, x + 1, y + 1, width, height,
@@ -1943,33 +1993,61 @@ public class DefaultThemePainter : IThemePainter
 				switch (button)
 				{
 					case ScrollButton.Up:
-					{
-						DrawGlyph(graphics, x, y, width, height,
+					{      
+						if(width<=20)  // if there is not enough space we draw a point
+						{
+							DrawGlyph(graphics, x, y, width - 3, height,
+								  Glyphs.up_arrow_bits,
+								  Glyphs.up_arrow_width / 2,
+								  Glyphs.up_arrow_height / 2, color);
+						
+						}
+
+						else 
+						{ 
+							DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.up_arrow_bits,
 								  Glyphs.up_arrow_width,
 								  Glyphs.up_arrow_height, color);
+						}
+						
 					}
 					break;
-
+					 
 					case ScrollButton.Down:
-					{
-						DrawGlyph(graphics, x, y, width, height,
+					{       
+						if(width<=20) // if there is not enough space we draw a point 
+						{
+							DrawGlyph(graphics, x, y, width - 3, height,
+								  Glyphs.up_arrow_bits,
+								  Glyphs.up_arrow_width / 2,
+								  Glyphs.up_arrow_height / 2, color);
+						
+						}
+						else
+						{
+							DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.down_arrow_bits,
 								  Glyphs.down_arrow_width,
 								  Glyphs.down_arrow_height, color);
+						}
 					}
 					break;
 
 					case ScrollButton.Left:
 					{
+					
+					     
+					   							
 						DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.left_arrow_bits,
 								  Glyphs.left_arrow_width,
-								  Glyphs.left_arrow_height, color);
+					     			  Glyphs.left_arrow_height, color);
+					        							
 					}
 					break;
 
-					case ScrollButton.Right:
+					case ScrollButton.Right:                                                
 					{
 						DrawGlyph(graphics, x, y, width, height,
 								  Glyphs.right_arrow_bits,
@@ -2092,12 +2170,58 @@ public class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a size grip.
-	[TODO]
 	public virtual void DrawSizeGrip
-				(Graphics graphics, Color backColor,
-				 int x, int y, int width, int height)
+				(Graphics graphics, Color backColor, Rectangle drawBounds)
 			{
-				// TODO
+				Pen light = new Pen(Color.White, 1);
+				Pen dark = new Pen(Color.Black, 1);
+
+				// Backfill accordingly
+				graphics.FillRectangle(new SolidBrush(backColor),
+							drawBounds.X, drawBounds.Y,
+							drawBounds.Width, drawBounds.Height);
+
+				// Top most line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 1,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1 ,
+						drawBounds.Top + 1
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top
+					);
+
+				// Middle line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 5,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 5
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left + 4,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 4
+					);
+
+				// Bottom line
+				graphics.DrawLine(SystemPens.ControlDark,
+						drawBounds.Left + 10,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top + 10
+					);
+				graphics.DrawLine(SystemPens.ControlLightLight,
+						drawBounds.Left + 9,
+						drawBounds.Height - 1,
+						drawBounds.Width - 1,
+						drawBounds.Top  + 9
+					);
 			}
 
 	// Draw a list box
@@ -2128,7 +2252,7 @@ public class DefaultThemePainter : IThemePainter
 				SolidBrush brush = new SolidBrush(ControlPaint.Light(color));
 				layoutRectangle.Offset(1.0f, 1.0f);
 				graphics.DrawString(s, font, brush, layoutRectangle, format);
-				brush.Color = ControlPaint.Dark(color);
+				brush.Color = ControlPaint.Dark(ControlPaint.Dark(ControlPaint.Dark(color)));
 				layoutRectangle.Offset(-1.0f, -1.0f);
 				graphics.DrawString(s, font, brush, layoutRectangle, format);
 				brush.Dispose();
@@ -2147,6 +2271,174 @@ public class DefaultThemePainter : IThemePainter
 					 x + width - 1, y + 1);
 			}
 
+	// Draw a track bar control.
+	public virtual void DrawTrackBar(Graphics graphics, Rectangle clientRect,
+					Rectangle barRect, Orientation orientation, bool enabled,
+					int ticks, TickStyle style)
+	{
+		int x = 0;
+		int lineheight = 4;
+		int linewidth = 4;
+
+		if(orientation == Orientation.Horizontal) 
+		{ // Draw horizontal
+			x = (clientRect.Width / ticks) + 1;
+
+			if(style == TickStyle.BottomRight) 
+			{
+				for(int cnt = x; cnt <= clientRect.Width; cnt += x) 
+				{
+					// right tick
+					DrawBorder3D(graphics,
+						clientRect.Left + cnt,
+						clientRect.Height / 2,
+						linewidth,
+						clientRect.Height / lineheight,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+			else if(style == TickStyle.TopLeft) 
+			{
+				for(int cnt = x; cnt <= clientRect.Width; cnt += x) 
+				{
+					// left tick
+					DrawBorder3D(graphics,
+						clientRect.Left + cnt,
+						clientRect.Top + clientRect.Height / lineheight,
+						linewidth,
+						clientRect.Height / lineheight,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+			else if(style == TickStyle.Both) 
+			{
+				for(int cnt = x; cnt <= clientRect.Width; cnt += x) 
+				{
+					// left tick
+					DrawBorder3D(graphics,
+						clientRect.Left + cnt,
+						clientRect.Height / 2 - lineheight,
+						linewidth,
+						lineheight * 2 + lineheight / 2,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+		}
+		else 
+		{ // Draw vertical
+			x = (clientRect.Height / ticks) + 1;
+
+			if(style == TickStyle.BottomRight) 
+			{
+				for(int cnt = x; cnt < clientRect.Height; cnt += x) 
+				{
+					// right tick
+					DrawBorder3D(graphics,
+						clientRect.Width / 2,
+						clientRect.Top + cnt,
+						clientRect.Width / linewidth,
+						lineheight,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+			else if(style == TickStyle.TopLeft) 
+			{
+				for(int cnt = x; cnt < clientRect.Height; cnt += x) 
+				{
+					// left tick
+					DrawBorder3D(graphics,
+						clientRect.Left + clientRect.Width / linewidth,
+						clientRect.Top + cnt,
+						clientRect.Width / linewidth,
+						lineheight,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+			else if(style == TickStyle.Both) 
+			{
+				for(int cnt = x; cnt < clientRect.Height; cnt += x) 
+				{
+					// left tick
+					DrawBorder3D(graphics,
+						clientRect.Width / 2 - linewidth,
+						clientRect.Top + cnt,
+						linewidth * 2 + linewidth / 2,
+						lineheight,
+						SystemColors.ControlText, SystemColors.Control,
+						Border3DStyle.Sunken, Border3DSide.All);
+				}
+			}
+		}
+
+		// draw end lines if we have a tickstyle
+		if(style != TickStyle.None) 
+		{
+			if(orientation == Orientation.Horizontal) 
+			{
+				// Draw middle bar
+				DrawBorder3D(graphics,
+					clientRect.Left, (clientRect.Height / 2) - (lineheight / 2),
+					clientRect.Width, lineheight,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+
+				// left vertical bar
+				DrawBorder3D(graphics,
+					clientRect.Left, clientRect.Top,
+					linewidth, clientRect.Height,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+
+				// right vertical bar
+				DrawBorder3D(graphics,
+					clientRect.Right - linewidth, clientRect.Top,
+					linewidth, clientRect.Height,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+			}
+			else 
+			{
+				// Draw middle bar
+				DrawBorder3D(graphics,
+					clientRect.Width / 2 - 1, clientRect.Y,
+					linewidth, clientRect.Height,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+
+				// top horizontal bar
+				DrawBorder3D(graphics,
+					clientRect.Left, clientRect.Top,
+					clientRect.Width, lineheight,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+
+				// bottom horizontal bar
+				DrawBorder3D(graphics,
+					clientRect.Left, clientRect.Bottom - lineheight,
+					clientRect.Width, lineheight,
+					SystemColors.ControlText, SystemColors.Control,
+					Border3DStyle.Sunken, Border3DSide.All);
+			}
+		}
+
+		// draw the button
+		if (((barRect.Height > 0) ||
+			(barRect.Width > 0)) && barRect.IntersectsWith(clientRect))
+		{
+			DrawButton(graphics,
+				barRect.X, barRect.Y,
+				barRect.Width,
+				barRect.Height,
+				(enabled ? ButtonState.Normal : ButtonState.Inactive),
+				SystemColors.ControlText, SystemColors.Control,
+				false);
+		}
+	}
 }; // class DefaultThemePainter
 
 }; // namespace System.Windows.Forms.Themes
