@@ -599,13 +599,15 @@ class XmlDocument : XmlNode
 				return new XmlDeclaration
 					(placeholder, version, encoding, standalone);
 			}
-	
+
+#if CONFIG_XPATH
 	// NOTE: should be protected internal virtual 
 	// but for ml-pnet's System.Data the internal modifier has to be removed
 	protected virtual XPathNavigator CreateNavigator(XmlNode node)
 			{
 				return node.CreateNavigator();
 			}
+#endif
 
 	// Get an element by identifier.
 	[TODO]
@@ -632,8 +634,21 @@ class XmlDocument : XmlNode
 	[TODO]
 	public virtual XmlNode ImportNode(XmlNode node, bool deep)
 			{
-				// TODO
-				return null;
+				// TODO: this may not be the best way to do this.
+				// xml namespaces, dtds and other interesting things
+				// have to be taken care of
+				XmlNode imported = this.CreateNode(node.NodeType,
+													node.Name,
+													node.NamespaceURI);
+				
+				if(deep)
+				{
+					foreach(XmlNode child in imported.ChildNodes)
+					{
+						ImportNode(child, deep);
+					}
+				}
+				return imported;
 			}
 
 	// Load XML into this document.
@@ -657,7 +672,7 @@ class XmlDocument : XmlNode
 	public virtual void Load(XmlReader reader)
 			{
 				RemoveAll();
-				if(!BuildStructure((XmlTextReader)reader))
+				if(!BuildStructure(reader))
 				{
 					throw new XmlException(S._("XmlException_NoXml"));
 				}
@@ -676,7 +691,7 @@ class XmlDocument : XmlNode
 			}
 
 	// Used by Load/LoadXml methods to do the DOM structure
-	private bool BuildStructure(XmlTextReader reader)
+	private bool BuildStructure(XmlReader reader)
 			{
 				// check if we should read in a full document
 				bool complete = (reader.Depth == 0);
@@ -746,7 +761,7 @@ class XmlDocument : XmlNode
 			}
 
 	// Used by the ReadNode method for recursively building a DOM subtree.
-	private XmlNode ReadNodeInternal(XmlReader r)
+	internal XmlNode ReadNodeInternal(XmlReader r)
 			{
 				switch(r.NodeType)
 				{
@@ -998,7 +1013,7 @@ class XmlDocument : XmlNode
 					return null;
 				}
 
-				XmlNode result;
+				XmlNode result = null;
 				switch(reader.NodeType)
 				{
 					case XmlNodeType.Attribute:
