@@ -7,9 +7,14 @@
 // 		(c) 2002
 
 using System;
+using System.ComponentModel;
 
 namespace System.Text.RegularExpressions {
-	
+	/* I'm just guessing that this is the correct place for this
+	 * attribute, and that the option is correct.  It shuts up
+	 * CorCompare for this undocumented class.
+	 */
+	[EditorBrowsable (EditorBrowsableState.Never)]
 	public abstract class RegexRunner {
 		// constructor
 	
