@@ -42,6 +42,10 @@ public class TopLevelWindow : InputOutputWidget
 	internal bool maximized;
 	internal bool hasPrimaryFocus;
 	internal bool reparented;
+	internal bool sticky;
+	internal bool shaded;
+	internal bool hidden;
+	internal bool fullScreen;
 	private bool firstMapDone;
 	private IntPtr keyBuffer;
 	internal InputOnlyWidget focusWidget;
@@ -146,6 +150,10 @@ public class TopLevelWindow : InputOutputWidget
 				this.maximized = false;
 				this.hasPrimaryFocus = false;
 				this.reparented = false;
+				this.sticky = false;
+				this.shaded = false;
+				this.hidden = false;
+				this.fullScreen = false;
 				this.keyBuffer = IntPtr.Zero;
 				this.focusWidget = this;
 				this.defaultFocus = null;
@@ -419,6 +427,42 @@ public class TopLevelWindow : InputOutputWidget
 						     XBool.False);
 				}
 
+				// Determine if we should stick in a fixed position
+				if((otherHints & OtherHints.Sticky) != 0)
+				{
+					atoms[numAtoms++] =
+						(Xlib.Xlong)Xlib.XInternAtom
+							(display, "_NET_WM_STATE_STICKY",
+						    XBool.False);
+				}
+
+				// Determine if we should shade
+				if((otherHints & OtherHints.Shaded) != 0)
+				{
+					atoms[numAtoms++] =
+						(Xlib.Xlong)Xlib.XInternAtom
+							(display, "_NET_WM_STATE_SHADED",
+						    XBool.False);
+				}
+
+				// Determine if we should hide
+				if((otherHints & OtherHints.Hidden) != 0)
+				{
+					atoms[numAtoms++] =
+						(Xlib.Xlong)Xlib.XInternAtom
+							(display, "_NET_WM_STATE_HIDDEN",
+						    XBool.False);
+				}
+
+				// Determine if we should go full screen
+				if((otherHints & OtherHints.FullScreen) != 0)
+				{
+					atoms[numAtoms++] =
+						(Xlib.Xlong)Xlib.XInternAtom
+							(display, "_NET_WM_STATE_FULLSCREEN",
+						    XBool.False);
+				}
+
 				// Determine if the window should be maximized by default.
 				if(maximized)
 				{
@@ -1192,6 +1236,41 @@ public class TopLevelWindow : InputOutputWidget
 									 "_NET_WM_WINDOW_TYPE_DIALOG",
 									 XBool.False);
 							}
+							else if((value & OtherHints.Desktop) != 0)
+							{
+								type = Xlib.XInternAtom
+									(display,
+									 "_NET_WM_WINDOW_TYPE_DESKTOP",
+									 XBool.False);
+							}
+							else if((value & OtherHints.Dock) != 0)
+							{
+								type = Xlib.XInternAtom
+									(display,
+									 "_NET_WM_WINDOW_TYPE_DOCK",
+									 XBool.False);
+							}
+							else if((value & OtherHints.Menu) != 0)
+							{
+								type = Xlib.XInternAtom
+									(display,
+									 "_NET_WM_WINDOW_TYPE_MENU",
+									 XBool.False);
+							}
+							else if((value & OtherHints.Splash) != 0)
+							{
+								type = Xlib.XInternAtom
+									(display,
+									 "_NET_WM_WINDOW_TYPE_SPLASH",
+									 XBool.False);
+							}
+							else if((value & OtherHints.TopMost) != 0)
+							{
+								type = Xlib.XInternAtom
+									(display,
+									 "_NET_WM_STATE_ABOVE",
+									 XBool.False);
+							}
 							else
 							{
 								type = Xlib.XInternAtom
@@ -1199,9 +1278,19 @@ public class TopLevelWindow : InputOutputWidget
 									 "_NET_WM_WINDOW_TYPE_NORMAL",
 									 XBool.False);
 							}
-							XAtom wmType = Xlib.XInternAtom
-								(display, "_NET_WM_WINDOW_TYPE",
-								 XBool.False);
+							XAtom wmType;
+							if ((value & OtherHints.TopMost) != 0)
+							{
+								wmType = Xlib.XInternAtom
+									(display, "_NET_WM_STATE",
+									XBool.False);
+							}
+							else
+							{
+								wmType = Xlib.XInternAtom
+									(display, "_NET_WM_WINDOW_TYPE",
+									XBool.False);
+							}
 							XAtom wmAtom = Xlib.XInternAtom
 								(display, "ATOM", XBool.False);
 							Xlib.Xlong[] data = new Xlib.Xlong [2];
@@ -1624,6 +1713,34 @@ public class TopLevelWindow : InputOutputWidget
 				}
 			}
 
+	/// <summary>
+	/// <para>Move this widget to above one of its siblings.</para>
+	/// </summary>
+	///
+	/// <param name="sibling">
+	/// <para>The sibling to move this widget above.</para>
+	/// </param>
+	public override void MoveToAbove(Widget sibling)
+			{
+				// TODO: support this in non-top-level mode
+				throw new XInvalidOperationException
+					(S._("X_NonTopLevelOperation"));
+			}
+
+	/// <summary>
+	/// <para>Move this widget to above one of its siblings.</para>
+	/// </summary>
+	///
+	/// <param name="sibling">
+	/// <para>The sibling to move this widget below.</para>
+	/// </param>
+	public override void MoveToBelow(Widget sibling)
+			{
+				// TODO: support this in non-top-level mode
+				throw new XInvalidOperationException
+					(S._("X_NonTopLevelOperation"));
+			}
+
 	// Detect that this top-level window has gained the primary focus.
 	private void PrimaryFocusIn()
 			{
@@ -2074,6 +2191,36 @@ public class TopLevelWindow : InputOutputWidget
 					}
 					break;
 
+				case Xsharp.Events.EventType.ButtonRelease:
+					{
+						// Handle mouse wheel events.
+
+						// Sanity check
+						if ((xevent.xbutton.button != ButtonName.Button4) &&
+						    (xevent.xbutton.button != ButtonName.Button5))
+							return;
+
+						// Dispatch the event.
+						widget = focusWidget;
+						while(widget != null)
+						{
+							io = (widget as InputOnlyWidget);
+							if (io != null)
+							{
+								if (io.DispatchWheelEvent (ref xevent))
+								{
+									break;
+								}
+							}
+							if (widget == this)
+							{
+								break;
+							}
+							widget = widget.Parent;
+						}
+					}
+					break;
+
 				case Xsharp.Events.EventType.ConfigureNotify:
 					{
 						// The window manager may have caused us to move/resize.
