@@ -2,6 +2,7 @@
  * NumberParser.cs - Implementation of "System.Private.NumberParser".
  *
  * Copyright (C) 2001  Southern Storm Software, Pty Ltd.
+ * Copyright (C) 2004  Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -74,6 +75,16 @@ internal sealed class NumberParser
 			throw new FormatException(_("Format_Integer"));
 		}
 
+		// Accept "0x" or "0X" for radix 16.
+		if (radix == 16 && posn <= len - 2 && value[posn] == '0')
+		{
+			ch = value[posn+1];
+			if (ch == 'x' || ch == 'X')
+			{
+				posn += 2;
+			}
+		}
+
 		// Parse the main part of the number.
 		low = 0;
 		high = 0;
@@ -154,7 +165,21 @@ internal sealed class NumberParser
 		return noOverflow;
 	}
 
-	public static int StringToInt32(String value, int radix, int ovfValue)
+	private static void NegateIfSignBit(ref ulong magnitude, ref bool sign,
+										ulong signbit, ulong mask)
+	{
+		// unpredictable if sign not currently positive
+		if (!sign && (magnitude & signbit) != 0ul)
+		{
+			// (~x)+1 => negate in two's complement, but only negating
+			// mask bits (which should be 0, or overflow)
+			magnitude = (magnitude ^ mask) + 1;
+			sign = true;
+		}
+	}
+
+	public static int StringToInt32(String value, int radix, int ovfValue,
+									uint signbit, uint mask)
 	{
 		ulong result;
 		bool sign;
@@ -162,6 +187,10 @@ internal sealed class NumberParser
 		// Parse the number.
 		if(StringToNumber(value, radix, out result, out sign))
 		{
+			if (radix != 10)
+			{
+				NegateIfSignBit(ref result, ref sign, signbit, mask);
+			}
 			if(!sign)
 			{
 				if(result <= 2147483647)
@@ -229,6 +258,11 @@ internal sealed class NumberParser
 		// Parse the number.
 		if(StringToNumber(value, radix, out result, out sign))
 		{
+			if (radix != 10)
+			{
+				NegateIfSignBit (ref result, ref sign,
+								 0x8000000000000000, UInt64.MaxValue);
+			}
 			if(!sign)
 			{
 				if(result <= 9223372036854775807)
@@ -291,21 +325,30 @@ internal sealed class NumberParser
 		uint digit, low, high;
 		ulong tempa, tempb;
 		bool noOverflow = true;
-
+		int t;
+		
 		// Validate the parameters.
 		if(s == null)
 		{
 			throw new ArgumentNullException("s");
 		}
 
+		while((t = s.IndexOf('\0')) != -1)
+		{
+			string tmp = s.Remove(t, 1);
+			s = tmp;
+		}
+
 		// Get the number format information to use.
 		nfi = NumberFormatInfo.GetInstance(nfi);
 		String posSign = nfi.PositiveSign;
 		String negSign = nfi.NegativeSign;
 		String currency = nfi.CurrencySymbol;
-		String thousandsSep1 = nfi.CurrencyGroupSeparator;
-		String thousandsSep2 = nfi.NumberGroupSeparator;
-
+		
+		String thousandsSep;
+		if( (style & NumberStyles.Float) != 0 ) thousandsSep = nfi.NumberGroupSeparator;
+		else                                    thousandsSep = nfi.CurrencyGroupSeparator;
+		
 		// Skip leading white space.
 		posn = 0;
 		len = s.Length;
@@ -355,8 +398,7 @@ internal sealed class NumberParser
 				++posn;
 			}
 			else if((style & NumberStyles.AllowThousands) != 0 &&
-					(ch == thousandsSep1[0] ||
-					 ch == thousandsSep2[0]))
+					 ch == thousandsSep[0] )
 			{
 				++posn;
 			}
@@ -406,8 +448,7 @@ internal sealed class NumberParser
 					digit = (uint)(ch - 'a' + 10);
 				}
 				else if((style & NumberStyles.AllowThousands) != 0 &&
-						(ch == thousandsSep1[0] ||
-						 ch == thousandsSep2[0]))
+						ch == thousandsSep[0] )
 				{
 					// Ignore thousands separators in the string.
 					++posn;
@@ -463,8 +504,7 @@ internal sealed class NumberParser
 					digit = (uint)(ch - '0');
 				}
 				else if((style & NumberStyles.AllowThousands) != 0 &&
-						(ch == thousandsSep1[0] ||
-						 ch == thousandsSep2[0]))
+						ch == thousandsSep[0])
 				{
 					// Ignore thousands separators in the string.
 					++posn;
@@ -547,6 +587,11 @@ internal sealed class NumberParser
 					lastWasWhite = true;
 					++posn;
 				}
+				else if(ch == '\0')
+				{
+					lastWasWhite = false;
+					++posn;
+				}
 				else
 				{
 					break;
@@ -1164,14 +1209,23 @@ internal sealed class NumberParser
 				// check for groupseparator
 				if((style & NumberStyles.AllowThousands) != 0)
 				{
-					if(!CheckString(ref sb, nfi.NumberGroupSeparator,
-									ref start, end))
-					{
-						if(!CheckString(ref sb, nfi.CurrencyGroupSeparator,
-										ref start, end))
+					if( (style & NumberStyles.Float) != 0 ) {
+						
+						if(!CheckString(ref sb, nfi.NumberGroupSeparator,
+								ref start, end))
 						{
 							break;
 						}
+						
+					}
+					else {
+					
+						if(!CheckString(ref sb, nfi.CurrencyGroupSeparator,
+								ref start, end))
+						{
+							break;
+						}
+					
 					}
 				}
 				else
@@ -1257,6 +1311,16 @@ internal sealed class NumberParser
 		char[] str = s.ToCharArray();
 		int stridx = 0;
 		int end = str.Length - 1;
+		int t;
+		
+		while((t = s.IndexOf('\0')) != -1)
+		{
+			string tmp = s.Remove(t, 1);
+			s = tmp;
+		}
+
+		str = s.ToCharArray();
+		end = str.Length - 1;
 
 		// skip whitespaces and handle currency symbol and parenthesis
 		SkipWhiteSpace(ref str, nfi.CurrencySymbol, style, ref stridx, ref end,
@@ -1437,11 +1501,18 @@ internal sealed class NumberParser
 		int expDigits = 0;
 		ulong expValue = 0;
 		double result = 0;
+		int t;
 
 		//  Double does not parse hex numbers
 		if ((style & NumberStyles.AllowHexSpecifier) != 0)
 			throw new FormatException(_("Format_HexNotSupported"));
 
+		while((t = s.IndexOf('\0')) != -1)
+		{
+			string tmp = s.Remove(t, 1);
+			s = tmp;
+		}
+
 		char[] str = s.ToCharArray();
 		end = str.Length - 1;
 
