@@ -27,6 +27,9 @@ namespace System.Runtime.InteropServices
 [ComVisible(false)]
 [Serializable]
 [Flags]
+#if CONFIG_FRAMEWORK_1_2
+[Obsolete("Use the class in System.Runtime.InteropServices.ComTypes instead")]
+#endif
 public enum IMPLTYPEFLAGS
 {
 	IMPLTYPEFLAG_FDEFAULT       = 0x0001,
